<?php
namespace Teknisa\MAN\Factories;

use Doctrine\DBAL\Connection;
use Zeedhi\Framework\DB\StoredProcedure\Param;
use Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;

class RelatoriosSaldoConsumidorFactory {
    protected $connection;

    public function __construct(Connection $connection) {
        $this->connection = $connection;
    }

    public function findSaldoNegativoConsumidorAtual($conditions) {
        $params = [
            'CDCLIENTE'     => $conditions['CDCLIENTE'],
            'TCDCONSUMIDOR' => $conditions['TCDCONSUMIDOR'],
            'TCDTIPOCONS'   => $conditions['TCDTIPOCONS'],
            'TCDFAMILISALD' => $conditions['TCDFAMILISALD'],
            'CDCONSUMIDOR'  => $conditions['CDCONSUMIDOR'],
            'CDTIPOCONS'    => $conditions['CDTIPOCONS'],
            'CDFAMILISALD'  => $conditions['CDFAMILISALD']
        ];
        
        $sql = "
            SELECT 
                CLI.NRINSJURCLIE,
                CLI.NMFANTCLIE  ,
                CLI.IDTPIJURCLIE,
                C.CDIDCONSUMID,
                C.NMCONSUMIDOR  ,
                T.CDTIPOCONS    , 
                DECODE (TC.NMTIPOCONSCLI, '', T.NMTIPOCONS, TC.NMTIPOCONSCLI)AS NMTIPOCONS,
                SUM(SC.VRSALDCONFAM) AS VRSALDCONFAM
            FROM
                CLIENTE CLI, FAMILIASALDO FS, CONSUMIDOR C, 
                SALDOCONS SC, TIPOCONS T, TIPOCONSCLI TC
            WHERE
                (C.CDCLIENTE    = :CDCLIENTE   ) AND
                ((INSTR(:CDCONSUMIDOR, '_'||C.CDCONSUMIDOR||'_' ) <> 0)  OR (:TCDCONSUMIDOR = 'T')) AND
                ((INSTR(:CDTIPOCONS  , '_'||C.CDTIPOCONS||'_'   ) <> 0)  OR (:TCDTIPOCONS   = 'T')) AND
                (SC.CDCLIENTE    = C.CDCLIENTE  ) AND
                (SC.CDCONSUMIDOR = C.CDCONSUMIDOR ) AND
                ((INSTR(:CDFAMILISALD, '_'||SC.CDFAMILISALD||'_') <> 0)  OR (:TCDFAMILISALD = 'T')) AND
                (SC.CDCLIENTE    = CLI.CDCLIENTE  ) AND
                (SC.CDFAMILISALD = FS.CDFAMILISALD) AND
                (C.CDTIPOCONS    = T.CDTIPOCONS   ) AND
                (SC.VRSALDCONFAM < 0) AND
                (C.CDCLIENTE  = TC.CDCLIENTE(+))   AND
                (C.CDTIPOCONS = TC.CDTIPOCONS(+))
            GROUP BY
                CLI.NRINSJURCLIE,
                CLI.NMFANTCLIE  ,
                CLI.IDTPIJURCLIE,
                C.CDIDCONSUMID,
                C.NMCONSUMIDOR  ,
                T.CDTIPOCONS    , 
                DECODE (TC.NMTIPOCONSCLI, '', T.NMTIPOCONS, TC.NMTIPOCONSCLI)
            ORDER BY 
                T.CDTIPOCONS
        ";
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        
        return $std->fetchAll();
    }

    public function findSaldoPositivoConsumidorAtual($conditions) {
        $params = [
            'CDCLIENTE'     => $conditions['CDCLIENTE'],
            'TCDCONSUMIDOR' => $conditions['TCDCONSUMIDOR'],
            'TCDTIPOCONS'   => $conditions['TCDTIPOCONS'],
            'TCDFAMILISALD' => $conditions['TCDFAMILISALD'],
            'CDCONSUMIDOR'  => $conditions['CDCONSUMIDOR'],
            'CDTIPOCONS'    => $conditions['CDTIPOCONS'],
            'CDFAMILISALD'  => $conditions['CDFAMILISALD']
        ];
        
        $sql = "
            SELECT 
                CLI.NRINSJURCLIE,
                CLI.NMFANTCLIE  ,
                CLI.IDTPIJURCLIE,
                C.NMCONSUMIDOR  ,
                T.CDTIPOCONS    , 
                DECODE (TC.NMTIPOCONSCLI, '', T.NMTIPOCONS, TC.NMTIPOCONSCLI)AS NMTIPOCONS,
                C.CDIDCONSUMID,
                SUM(SC.VRSALDCONFAM) AS VRSALDCONFAM
                FROM
                CLIENTE CLI, FAMILIASALDO FS, CONSUMIDOR C, 
                SALDOCONS SC, TIPOCONS T, TIPOCONSCLI TC
            WHERE
                (C.CDCLIENTE    = :CDCLIENTE   ) AND
                ((INSTR(:CDCONSUMIDOR, '_'||C.CDCONSUMIDOR||'_' ) <> 0)  OR (:TCDCONSUMIDOR = 'T')) AND
                ((INSTR(:CDTIPOCONS  , '_'||C.CDTIPOCONS||'_'   ) <> 0)  OR (:TCDTIPOCONS   = 'T')) AND
                (SC.CDCLIENTE    = C.CDCLIENTE  ) AND
                (SC.CDCONSUMIDOR = C.CDCONSUMIDOR ) AND
                ((INSTR(:CDFAMILISALD, '_'||SC.CDFAMILISALD||'_') <> 0)  OR (:TCDFAMILISALD = 'T')) AND
                (SC.CDCLIENTE    = CLI.CDCLIENTE  ) AND
                (SC.CDFAMILISALD = FS.CDFAMILISALD) AND
                (C.CDTIPOCONS    = T.CDTIPOCONS   ) AND
                (SC.VRSALDCONFAM > 0) AND
                (C.CDCLIENTE  = TC.CDCLIENTE(+))   AND
                (C.CDTIPOCONS = TC.CDTIPOCONS(+))
            GROUP BY
                CLI.NRINSJURCLIE,
                CLI.NMFANTCLIE  ,
                CLI.IDTPIJURCLIE,
                C.NMCONSUMIDOR  ,
                T.CDTIPOCONS    , 
                DECODE (TC.NMTIPOCONSCLI, '', T.NMTIPOCONS, TC.NMTIPOCONSCLI),
                C.CDIDCONSUMID
            ORDER BY 
                T.CDTIPOCONS
        ";
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        
        return $std->fetchAll();
    }

    public function findSaldoConsumidorPeriodo($conditions) {
        $params = [
            'CDCLIENTE'     => $conditions['CDCLIENTE'],
            'TCDCONSUMIDOR' => $conditions['TCDCONSUMIDOR'],
            'TCDTIPOCONS'   => $conditions['TCDTIPOCONS'],
            'TCDFAMILISALD' => $conditions['TCDFAMILISALD'],
            'CDCONSUMIDOR'  => $conditions['CDCONSUMIDOR'],
            'CDTIPOCONS'    => $conditions['CDTIPOCONS'],
            'CDFAMILISALD'  => $conditions['CDFAMILISALD'],
            'DTEXTRATOINI'  => $conditions['DTEXTRATOINI'],
            'DTEXTRATOFIN'  => $conditions['DTEXTRATOFIN']
        ];
        
        $sql = "
            SELECT 
              :DTEXTRATOINI as DATAINI,
              :DTEXTRATOFIN as DATAFIN,
              E.DTMOVEXTCONS,
              CLI.NRINSJURCLIE,
              CLI.NMFANTCLIE  ,
              CLI.IDTPIJURCLIE,
              E.CDFAMILISALD ,
              FS.NMFAMILISALD , 
              C.CDIDCONSUMID  , 
              C.NMCONSUMIDOR  ,
              T.CDTIPOCONS    , 
              DECODE (TC.NMTIPOCONSCLI, '', T.NMTIPOCONS, TC.NMTIPOCONSCLI)AS NMTIPOCONS,
              E.IDTPMOVEXT,
              DECODE (E.IDTPMOVEXT, 'C', E.VRMOVEXTCONS, - 1 * E.VRMOVEXTCONS) AS VRMOVEXTCONS,
              E.VRSALDCONEXT,
              C.CDIDCONSUMID||' - '||C.NMCONSUMIDOR AS CDNMCONSUMIDOR,
              E.CDFAMILISALD||' - '||FS.NMFAMILISALD AS CDNMFAMILISALD
            FROM
              CLIENTE CLI, FAMILIASALDO FS, CONSUMIDOR C, TIPOCONS T, EXTRATOCONS E, TIPOCONSCLI TC
            WHERE
              (C.CDCLIENTE    = :CDCLIENTE   ) AND
              ((INSTR(:CDCONSUMIDOR, '_'||C.CDCONSUMIDOR||'_' ) <> 0)  OR (:TCDCONSUMIDOR = 'T')) AND
              ((INSTR(:CDTIPOCONS  , '_'||C.CDTIPOCONS||'_'   ) <> 0)  OR (:TCDTIPOCONS   = 'T')) AND
              (E.CDCLIENTE    = C.CDCLIENTE  ) AND
              (E.CDCONSUMIDOR = C.CDCONSUMIDOR ) AND
              ((INSTR(:CDFAMILISALD, '_'||E.CDFAMILISALD||'_') <> 0)  OR (:TCDFAMILISALD = 'T')) AND
              (E.CDCLIENTE    = CLI.CDCLIENTE  ) AND
              (E.CDFAMILISALD = FS.CDFAMILISALD) AND
              (C.CDTIPOCONS    = T.CDTIPOCONS   )AND
              (TO_DATE(E.DTMOVEXTCONS) BETWEEN :DTEXTRATOINI AND :DTEXTRATOFIN) AND
              (C.CDCLIENTE  = TC.CDCLIENTE(+))   AND
              (C.CDTIPOCONS = TC.CDTIPOCONS(+))
            ORDER BY 
              E.CDFAMILISALD,T.CDTIPOCONS, E.DTMOVEXTCONS, C.CDIDCONSUMID
        ";
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        
        return $std->fetchAll();
    }

    public function findSaldoNegativoConsumidorPeriodo($conditions) {
        $params = [
            'CDCLIENTE'     => $conditions['CDCLIENTE'],
            'TCDCONSUMIDOR' => $conditions['TCDCONSUMIDOR'],
            'TCDTIPOCONS'   => $conditions['TCDTIPOCONS'],
            'TCDFAMILISALD' => $conditions['TCDFAMILISALD'],
            'CDCONSUMIDOR'  => $conditions['CDCONSUMIDOR'],
            'CDTIPOCONS'    => $conditions['CDTIPOCONS'],
            'CDFAMILISALD'  => $conditions['CDFAMILISALD'],
            'DTEXTRATOINI'  => $conditions['DTEXTRATOINI'],
            'DTEXTRATOFIN'  => $conditions['DTEXTRATOFIN']
        ];
        
        $sql = "
            SELECT DISTINCT
              CLI.NRINSJURCLIE,
              CLI.NMFANTCLIE  ,
              CLI.IDTPIJURCLIE,
              E.CDCLIENTE     ,
              E.CDCONSUMIDOR  ,
              E.CDFAMILISALD,
              FS.NMFAMILISALD,
              E.DTMOVEXTCONS,
              C.NMCONSUMIDOR  ,
              T.CDTIPOCONS    , 
              DECODE (TC.NMTIPOCONSCLI, '', T.NMTIPOCONS, TC.NMTIPOCONSCLI)AS NMTIPOCONS,
              C.CDIDCONSUMID,
              :DTEXTRATOINI,
              :DTEXTRATOFIN,
              E.VRSALDCONEXT,
              E.CDFAMILISALD||' - '||FS.NMFAMILISALD AS CDNMFAMILISALD
            FROM
              CLIENTE CLI, FAMILIASALDO FS, CONSUMIDOR C, TIPOCONS T, EXTRATOCONS E, TIPOCONSCLI TC,
              (SELECT MAX(E.DTMOVEXTCONS) AS DTMOVEXTCONS,
              C.CDIDCONSUMID,
              C.CDTIPOCONS,
              E.CDCLIENTE,
              E.CDFAMILISALD,
              C.CDCONSUMIDOR
            FROM EXTRATOCONS E, CONSUMIDOR C
            WHERE (E.CDCLIENTE = :CDCLIENTE)
              AND (TO_DATE(E.DTMOVEXTCONS) BETWEEN :DTEXTRATOINI AND :DTEXTRATOFIN)
              AND ((INSTR(:CDCONSUMIDOR, '_'||C.CDCONSUMIDOR||'_' ) <> 0)  OR (:TCDCONSUMIDOR = 'T')) 
              AND ((INSTR(:CDTIPOCONS  , '_'||C.CDTIPOCONS||'_'   ) <> 0)  OR (:TCDTIPOCONS   = 'T')) 
              AND ((INSTR(:CDFAMILISALD, '_'||E.CDFAMILISALD||'_') <> 0)  OR (:TCDFAMILISALD = 'T'))
              AND (E.CDCLIENTE    = C.CDCLIENTE  )
              AND (E.CDCONSUMIDOR = C.CDCONSUMIDOR)
              AND (C.CDIDCONSUMID IS NOT NULL)
            GROUP BY
              C.CDIDCONSUMID,
              C.CDTIPOCONS,
              E.CDCLIENTE,
              E.CDFAMILISALD,
              C.CDCONSUMIDOR) A
            WHERE
              (E.DTMOVEXTCONS = A.DTMOVEXTCONS) AND
              (C.CDCLIENTE    = A.CDCLIENTE     ) AND
              (C.CDCONSUMIDOR = A.CDCONSUMIDOR  ) AND
              (C.CDIDCONSUMID = A.CDIDCONSUMID  ) AND
              (C.CDTIPOCONS   = A.CDTIPOCONS    ) AND
              (E.CDCLIENTE    = C.CDCLIENTE    ) AND
              (E.CDCONSUMIDOR = C.CDCONSUMIDOR ) AND
              (E.CDFAMILISALD = A.CDFAMILISALD  ) AND
              (E.CDCLIENTE    = CLI.CDCLIENTE  ) AND
              (E.CDFAMILISALD = FS.CDFAMILISALD) AND
              (C.CDTIPOCONS   = T.CDTIPOCONS   ) AND
              (C.CDCLIENTE  = TC.CDCLIENTE  (+)) AND
              (C.CDTIPOCONS = TC.CDTIPOCONS (+)) AND
              (E.VRSALDCONEXT < 0)
            ORDER BY 
              T.CDTIPOCONS, C.CDIDCONSUMID, C.NMCONSUMIDOR, E.CDFAMILISALD
        ";
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        
        return $std->fetchAll();
    }

    public function findSaldoPositivoConsumidorPeriodo($conditions) {
        $params = [
            'CDCLIENTE'     => $conditions['CDCLIENTE'],
            'TCDCONSUMIDOR' => $conditions['TCDCONSUMIDOR'],
            'TCDTIPOCONS'   => $conditions['TCDTIPOCONS'],
            'TCDFAMILISALD' => $conditions['TCDFAMILISALD'],
            'CDCONSUMIDOR'  => $conditions['CDCONSUMIDOR'],
            'CDTIPOCONS'    => $conditions['CDTIPOCONS'],
            'CDFAMILISALD'  => $conditions['CDFAMILISALD'],
            'DTEXTRATOINI'  => $conditions['DTEXTRATOINI'],
            'DTEXTRATOFIN'  => $conditions['DTEXTRATOFIN']
        ];
        
        $sql = "
            SELECT DISTINCT
              CLI.NRINSJURCLIE,
              CLI.NMFANTCLIE  ,
              CLI.IDTPIJURCLIE,
              E.CDCLIENTE     ,
              E.CDCONSUMIDOR  ,
              E.CDFAMILISALD,
              FS.NMFAMILISALD,
              E.DTMOVEXTCONS,
              C.NMCONSUMIDOR  ,
              T.CDTIPOCONS    , 
              DECODE (TC.NMTIPOCONSCLI, '', T.NMTIPOCONS, TC.NMTIPOCONSCLI)AS NMTIPOCONS,
              C.CDIDCONSUMID,
              :DTEXTRATOINI,
              :DTEXTRATOFIN,
              E.VRSALDCONEXT,
              E.CDFAMILISALD||' - '||FS.NMFAMILISALD AS CDNMFAMILISALD
            FROM
              CLIENTE CLI, FAMILIASALDO FS, CONSUMIDOR C, TIPOCONS T, EXTRATOCONS E, TIPOCONSCLI TC,
              (SELECT MAX(E.DTMOVEXTCONS) AS DTMOVEXTCONS,
              C.CDIDCONSUMID,
              C.CDTIPOCONS,
              E.CDCLIENTE,
              E.CDFAMILISALD,
              C.CDCONSUMIDOR
            FROM EXTRATOCONS E, CONSUMIDOR C
            WHERE (E.CDCLIENTE = :CDCLIENTE)
              AND (TO_DATE(E.DTMOVEXTCONS) BETWEEN :DTEXTRATOINI AND :DTEXTRATOFIN)
              AND ((INSTR(:CDCONSUMIDOR, '_'||C.CDCONSUMIDOR||'_' ) <> 0)  OR (:TCDCONSUMIDOR = 'T')) 
              AND ((INSTR(:CDTIPOCONS  , '_'||C.CDTIPOCONS||'_'   ) <> 0)  OR (:TCDTIPOCONS   = 'T')) 
              AND ((INSTR(:CDFAMILISALD, '_'||E.CDFAMILISALD||'_') <> 0)  OR (:TCDFAMILISALD = 'T'))
              AND (E.CDCLIENTE    = C.CDCLIENTE  )
              AND (E.CDCONSUMIDOR = C.CDCONSUMIDOR)
              AND (C.CDIDCONSUMID IS NOT NULL)
            GROUP BY
              C.CDIDCONSUMID,
              C.CDTIPOCONS,
              E.CDCLIENTE,
              E.CDFAMILISALD,
              C.CDCONSUMIDOR) A
            WHERE
              (E.DTMOVEXTCONS = A.DTMOVEXTCONS) AND
              (C.CDCLIENTE    = A.CDCLIENTE     ) AND
              (C.CDCONSUMIDOR = A.CDCONSUMIDOR  ) AND
              (C.CDIDCONSUMID = A.CDIDCONSUMID  ) AND
              (C.CDTIPOCONS   = A.CDTIPOCONS    ) AND
              (E.CDCLIENTE    = C.CDCLIENTE    ) AND
              (E.CDCONSUMIDOR = C.CDCONSUMIDOR ) AND
              (E.CDFAMILISALD = A.CDFAMILISALD  ) AND
              (E.CDCLIENTE    = CLI.CDCLIENTE  ) AND
              (E.CDFAMILISALD = FS.CDFAMILISALD) AND
              (C.CDTIPOCONS   = T.CDTIPOCONS   ) AND
              (C.CDCLIENTE  = TC.CDCLIENTE  (+)) AND
              (C.CDTIPOCONS = TC.CDTIPOCONS (+)) AND
              (E.VRSALDCONEXT > 0)
            ORDER BY 
              T.CDTIPOCONS, C.CDIDCONSUMID,C.NMCONSUMIDOR, E.CDFAMILISALD
        ";
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        
        return $std->fetchAll();
    }

    public function findConsumidoresSemMov($conditions) {
        $params = [
            'CDCLIENTE'     => $conditions['CDCLIENTE'],
            'TCDCONSUMIDOR' => $conditions['TCDCONSUMIDOR'],
            'TCDTIPOCONS'   => $conditions['TCDTIPOCONS'],
            'TCDFAMILISALD' => $conditions['TCDFAMILISALD'],
            'CDCONSUMIDOR'  => $conditions['CDCONSUMIDOR'],
            'CDTIPOCONS'    => $conditions['CDTIPOCONS'],
            'CDFAMILISALD'  => $conditions['CDFAMILISALD'],
            'DTEXTRATOINI'  => $conditions['DTEXTRATOINI'],
            'DTEXTRATOFIN'  => $conditions['DTEXTRATOFIN']
        ];
        
        $sql = "
        SELECT
          CLI.NRINSJURCLIE,
          CLI.NMFANTCLIE  ,
          CLI.IDTPIJURCLIE,
          C.CDIDCONSUMID  ,
          C.NMCONSUMIDOR  ,
          T.CDTIPOCONS    ,
          DECODE (TC.NMTIPOCONSCLI, '', T.NMTIPOCONS, TC.NMTIPOCONSCLI) AS NMTIPOCONS,
          NVL(SUM(S.VRSALDCONFAM),0) AS VRSALDCONFAM,
          :DTEXTRATOINI AS DATAINICIAL ,
          :DTEXTRATOFIN AS DATAFINAL
        FROM
          CLIENTE CLI ,
          CONSUMIDOR C,
          TIPOCONS T  ,
          SALDOCONS S ,
          TIPOCONSCLI TC
        WHERE
          (C.CDCLIENTE     = :CDCLIENTE) AND
          (C.CDCLIENTE     = S.CDCLIENTE   (+)) AND
          (C.CDCONSUMIDOR  = S.CDCONSUMIDOR(+)) AND
          (C.CDCLIENTE     = CLI.CDCLIENTE  ) AND
          (C.CDTIPOCONS    = T.CDTIPOCONS   ) AND
          (C.CDCLIENTE  = TC.CDCLIENTE(+))   AND
          (C.CDTIPOCONS = TC.CDTIPOCONS(+)) AND
          ((INSTR(:CDCONSUMIDOR, '_'||C.CDCONSUMIDOR||'_' ) <> 0)  OR (:TCDCONSUMIDOR = 'T')) AND
          ((INSTR(:CDFAMILISALD, '_'||CDFAMILISALD  ||'_') <> 0)  OR (:TCDFAMILISALD = 'T')) AND
          ((INSTR(:CDTIPOCONS  , '_'||C.CDTIPOCONS  ||'_') <> 0)  OR (:TCDTIPOCONS   = 'T')) AND
           C.CDCONSUMIDOR NOT IN (SELECT DISTINCT
                                    EX.CDCONSUMIDOR
                                  FROM
                                    EXTRATOCONS EX
                                  WHERE
                                    (EX.CDCLIENTE    = :CDCLIENTE ) AND
                                    (TO_DATE(EX.DTMOVEXTCONS) BETWEEN :DTEXTRATOINI AND :DTEXTRATOFIN))
        GROUP BY
          CLI.NRINSJURCLIE,
          CLI.NMFANTCLIE  ,
          CLI.IDTPIJURCLIE,
          C.CDIDCONSUMID  ,
          C.NMCONSUMIDOR  ,
          T.CDTIPOCONS    ,
          DECODE (TC.NMTIPOCONSCLI, '', T.NMTIPOCONS, TC.NMTIPOCONSCLI)
        ORDER BY
          CLI.NRINSJURCLIE, C.CDIDCONSUMID, T.CDTIPOCONS
        ";
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        
        return $std->fetchAll();
    }

    public function findExtratoFamiliaQry1($conditions) {
        $params = [
            'CDCLIENTE'     => $conditions['CDCLIENTE'],
            'TCDCONSUMIDOR' => $conditions['TCDCONSUMIDOR'],
            'TCDTIPOCONS'   => $conditions['TCDTIPOCONS'],
            'TCDFAMILISALD' => $conditions['TCDFAMILISALD'],
            'CDCONSUMIDOR'  => $conditions['CDCONSUMIDOR'],
            'CDTIPOCONS'    => $conditions['CDTIPOCONS'],
            'CDFAMILISALD'  => $conditions['CDFAMILISALD'],
            'DTEXTRATOINI'  => $conditions['DTEXTRATOINI'],
            'DTEXTRATOFIN'  => $conditions['DTEXTRATOFIN']
        ];
        
        $sql = "
            SELECT
               CLI.NRINSJURCLIE,
               CLI.NMFANTCLIE  ,
               CLI.IDTPIJURCLIE,
               FS.CDFAMILISALD ,
               FS.NMFAMILISALD ,
               C.CDIDCONSUMID  ,
               C.NMCONSUMIDOR  ,
               T.CDTIPOCONS    ,
               DECODE (TC.NMTIPOCONSCLI, '', T.NMTIPOCONS, TC.NMTIPOCONSCLI)AS NMTIPOCONS ,
               EX.CDCAIXA      ,
               EX.CDFILIAL     ,
               EX.NRSEQVENDA   ,
               EX.VRSALDCONEXT ,
               EX.DTMOVEXTCONS ,
               DECODE (EX.DSOPEEXTCONS, (SUBSTR(EX.DSOPEEXTCONS, 1,20) || ' - EfeCred'),
                      ('Credito Via Site Doc. Nº' || SUBSTR(EX.DSOPEEXTCONS,11,10)),EX.DSOPEEXTCONS)AS DSOPEEXTCONS,
               EX.VRMOVEXTCONS ,
               EX.IDTPMOVEXT   ,
               :DTEXTRATOINI AS DATAINICIAL ,
               :DTEXTRATOFIN AS DATAFINAL,
               TO_CHAR(EX.DTMOVEXTCONS, 'DD/MM/YYYY') AS DATA,
               TO_CHAR(EX.DTMOVEXTCONS, 'HH24:MI:SS') AS HORA,
               NVL(SLDOA.SALDANT,0) AS SALDANT,
               SE.NMSERVICO ,
               VE.DTHRREALCONCONS
            FROM
               CLIENTE CLI, FAMILIASALDO FS, CONSUMIDOR C,
               EXTRATOCONS EX, TIPOCONS T, TIPOCONSCLI TC,
               VENDA VE, SERVICO SE,
               (SELECT
                   E.CDCONSUMIDOR,
                   E.CDFAMILISALD,
                   E.VRSALDCONEXT AS SALDANT
                FROM
                   EXTRATOCONS E,
                   (SELECT
                       E.CDCLIENTE,
                       E.CDCONSUMIDOR,
                       E.CDFAMILISALD,
                       E.DTMOVEXTCONS,
                       MAX(E.NRSEQMOVEXT) AS NRSEQMOVEXT
                    FROM
                       EXTRATOCONS E,
                       (SELECT
                           E.CDCLIENTE,
                           E.CDCONSUMIDOR,
                           E.CDFAMILISALD,
                           MAX(E.DTMOVEXTCONS) AS DTMOVEXTCONS
                        FROM EXTRATOCONS E, CONSUMIDOR C
                        WHERE
                           (C.CDCLIENTE     = :CDCLIENTE)                                                        AND
                           ((INSTR(:CDCONSUMIDOR, '_'||C.CDCONSUMIDOR||'_' ) <> 0)  OR (:TCDCONSUMIDOR = 'T')) AND
                           ((INSTR(:CDTIPOCONS  , '_'||C.CDTIPOCONS||'_'   ) <> 0)  OR (:TCDTIPOCONS   = 'T')) AND
                           (E.CDCLIENTE    = C.CDCLIENTE)                                                          AND
                           (E.CDCONSUMIDOR = C.CDCONSUMIDOR)                                                       AND
                           ((INSTR(:CDFAMILISALD, '_'||E.CDFAMILISALD||'_') <> 0)   OR (:TCDFAMILISALD = 'T')) AND
                           (TO_DATE(E.DTMOVEXTCONS) < :DTEXTRATOINI)
                        GROUP BY
                           E.CDCLIENTE,
                           E.CDCONSUMIDOR,
                           E.CDFAMILISALD
                       ) D
                    WHERE
                       (E.CDCLIENTE    = D.CDCLIENTE)    AND
                       (E.CDCONSUMIDOR = D.CDCONSUMIDOR) AND
                       (E.CDFAMILISALD = D.CDFAMILISALD) AND
                       (E.DTMOVEXTCONS = D.DTMOVEXTCONS)
                    GROUP BY
                       E.CDCLIENTE,
                       E.CDCONSUMIDOR,
                       E.CDFAMILISALD,
                       E.DTMOVEXTCONS
                   ) N
                WHERE
                   (E.CDCLIENTE    = N.CDCLIENTE)    AND
                   (E.CDCONSUMIDOR = N.CDCONSUMIDOR) AND
                   (E.CDFAMILISALD = N.CDFAMILISALD) AND
                   (E.DTMOVEXTCONS = N.DTMOVEXTCONS) AND
                   (E.NRSEQMOVEXT  = N.NRSEQMOVEXT)
               ) SLDOA
            WHERE
               (C.CDCLIENTE    = :CDCLIENTE   ) AND
               ((INSTR(:CDCONSUMIDOR, '_'||C.CDCONSUMIDOR||'_' ) <> 0)  OR (:TCDCONSUMIDOR = 'T')) AND
               ((INSTR(:CDTIPOCONS  , '_'||C.CDTIPOCONS||'_'   ) <> 0)  OR (:TCDTIPOCONS   = 'T')) AND
               (TO_DATE(EX.DTMOVEXTCONS) BETWEEN :DTEXTRATOINI AND :DTEXTRATOFIN) AND
               (EX.CDCLIENTE     = C.CDCLIENTE  ) AND
               (EX.CDCLIENTE     = CLI.CDCLIENTE  ) AND
               (EX.CDCONSUMIDOR  = C.CDCONSUMIDOR ) AND
               ((INSTR(:CDFAMILISALD, '_'||EX.CDFAMILISALD||'_') <> 0)  OR (:TCDFAMILISALD = 'T')) AND
               (EX.CDFAMILISALD  = FS.CDFAMILISALD) AND
               (C.CDTIPOCONS     = T.CDTIPOCONS   ) AND
               (EX.CDCONSUMIDOR  = SLDOA.CDCONSUMIDOR (+)) AND
               (EX.CDFAMILISALD  = SLDOA.CDFAMILISALD (+)) AND
               (C.CDCLIENTE      = TC.CDCLIENTE(+))   AND
               (C.CDTIPOCONS     = TC.CDTIPOCONS(+))  AND
               (VE.CDSERVFILI   = SE.CDSERVFILI  (+)) AND
               (VE.CDFILIAL     = SE.CDFILIAL    (+)) AND
               (EX.CDCLIENTE    = VE.CDCLIENTE   (+)) AND
               (EX.CDCONSUMIDOR = VE.CDCONSUMIDOR(+)) AND
               (EX.CDFILIAL     = VE.CDFILIAL    (+)) AND
               (EX.CDCAIXA      = VE.CDCAIXA     (+)) AND
               (EX.NRSEQVENDA   = VE.NRSEQVENDA  (+))
            ORDER BY
               T.CDTIPOCONS, C.CDIDCONSUMID, FS.CDFAMILISALD , EX.DTMOVEXTCONS, EX.NRSEQMOVEXT
        ";
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        
        return $std->fetchAll();
    }

    public function findExtratoFamiliaQry2($relFil) {
        $params = [
            'CDFILIAL'      => $relFil['CDFILIAL'],
            'CDCAIXA'       => $relFil['CDCAIXA'],
            'NRSEQVENDA'    => $relFil['NRSEQVENDA'],
            'IDTPMOVEXT'    => $relFil['IDTPMOVEXT'],
            'CDFAMILISALD'  => $relFil['CDFAMILISALD']
        ];
        
        $sql = "
            SELECT
               P.NMPRODUTO,
               NVL(SUM(IT.QTPRODVEND),0) AS QTDE,
               NVL(SUM((TRUNC(IT.QTPRODVEND*(IT.VRUNITVEND+IT.VRUNITVENDCL),2)) - (NVL(IT.VRDESITVEND,0) + NVL(IT.VRACRITVEND,0))),0) AS VRTOTAL
            FROM
               ITEMVENDA IT,
               PRODUTO P
            WHERE
               (IT.CDFILIAL     = :CDFILIAL    ) AND
               (IT.CDCAIXA      = :CDCAIXA     ) AND
               (IT.NRSEQVENDA   = :NRSEQVENDA  ) AND
               (IT.CDFAMILISALD = :CDFAMILISALD) AND
               (:IDTPMOVEXT     = 'V')           AND
               (IT.CDPRODUTO    = P.CDPRODUTO)
            GROUP BY
               IT.CDPRODUTO,
               P.NMPRODUTO
            ORDER BY
               P.NMPRODUTO
        ";
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        
        return $std->fetchAll();
    }

    public function findExtratoFamiliaQry3($conditions) {
        $params = [
            'CDFILIAL'      => $conditions['CDFILIAL'],
            'CDCAIXA'       => $conditions['CDCAIXA'],
            'NRSEQVENDA'    => $conditions['NRSEQVENDA'],
            'IDTPMOVEXT'    => $conditions['IDTPMOVEXT']
        ];
        
        $sql = "
            SELECT NVL(SUM(VRMOVIVEND),0) AS VRMOVIVEND
                FROM MOVCAIXA MO , TIPORECE TP
                WHERE
                   (MO.CDFILIAL     = :CDFILIAL  ) AND
                   (MO.CDCAIXA      = :CDCAIXA   ) AND
                   (MO.NRSEQVENDA   = :NRSEQVENDA) AND
                   (:IDTPMOVEXT     = 'V'        ) AND
                   (MO.CDTIPORECE = TP.CDTIPORECE) AND
                   (IDTIPORECE NOT IN ( '9','A'))
        ";
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        
        return $std->fetchAll();
    }

    public function findEstornoPorConsumidor($conditions) {
        $params = [
            'CDCLIENTE'     => $conditions['CDCLIENTE'],
            'TCDCONSUMIDOR' => $conditions['TCDCONSUMIDOR'],
            'TCDTIPOCONS'   => $conditions['TCDTIPOCONS'],
            'TCDFAMILISALD' => $conditions['TCDFAMILISALD'],
            'CDCONSUMIDOR'  => $conditions['CDCONSUMIDOR'],
            'CDTIPOCONS'    => $conditions['CDTIPOCONS'],
            'CDFAMILISALD'  => $conditions['CDFAMILISALD'],
            'DTEXTRATOINI'  => $conditions['DTEXTRATOINI'],
            'DTEXTRATOFIN'  => $conditions['DTEXTRATOFIN']
        ];
        $sql = "
        SELECT
           CLI.NRINSJURCLIE,
           CLI.NMFANTCLIE  ,
           CLI.IDTPIJURCLIE,
           FS.CDFAMILISALD ,
           FS.NMFAMILISALD ,
           C.CDIDCONSUMID  ,
           C.NMCONSUMIDOR  ,
           T.CDTIPOCONS    ,
           DECODE (TC.NMTIPOCONSCLI, '', T.NMTIPOCONS, TC.NMTIPOCONSCLI)AS NMTIPOCONS ,
           EX.CDCAIXA      ,
           EX.CDFILIAL     ,
           EX.NRSEQVENDA   ,
           EX.VRSALDCONEXT ,
           EX.DTMOVEXTCONS ,
           DECODE (EX.DSOPEEXTCONS, (SUBSTR(EX.DSOPEEXTCONS, 1,20) || ' - EfeCred'),
                  ('Credito Via Site Doc. Nº' || SUBSTR(EX.DSOPEEXTCONS,11,10)),EX.DSOPEEXTCONS)AS DSOPEEXTCONS,
           EX.VRMOVEXTCONS ,
           EX.IDTPMOVEXT   ,
           :DTEXTRATOINI AS DATAINICIAL ,
           :DTEXTRATOFIN AS DATAFINAL,
           TO_CHAR(EX.DTMOVEXTCONS, 'DD/MM/YYYY') AS DATA,
           TO_CHAR(EX.DTMOVEXTCONS, 'HH24:MI:SS') AS HORA,
           NVL(SLDOA.SALDANT,0) AS SALDANT,
           SE.NMSERVICO ,
           VE.DTHRREALCONCONS
        FROM
           CLIENTE CLI, FAMILIASALDO FS, CONSUMIDOR C,
           EXTRATOCONS EX, TIPOCONS T, TIPOCONSCLI TC,
           VENDA VE, SERVICO SE,
           (SELECT
               E.CDCONSUMIDOR,
               E.CDFAMILISALD,
               E.VRSALDCONEXT AS SALDANT
            FROM
               EXTRATOCONS E,
               (SELECT
                   E.CDCLIENTE,
                   E.CDCONSUMIDOR,
                   E.CDFAMILISALD,
                   E.DTMOVEXTCONS,
                   MAX(E.NRSEQMOVEXT) AS NRSEQMOVEXT
                FROM
                   EXTRATOCONS E,
                   (SELECT
                       E.CDCLIENTE,
                       E.CDCONSUMIDOR,
                       E.CDFAMILISALD,
                       MAX(E.DTMOVEXTCONS) AS DTMOVEXTCONS
                    FROM EXTRATOCONS E, CONSUMIDOR C
                    WHERE
                       (C.CDCLIENTE     = :CDCLIENTE)                                                        AND
                       ((INSTR(:CDCONSUMIDOR, '_'||C.CDCONSUMIDOR||'_' ) <> 0)  OR (:TCDCONSUMIDOR = 'T')) AND
                       ((INSTR(:CDTIPOCONS  , '_'||C.CDTIPOCONS||'_'   ) <> 0)  OR (:TCDTIPOCONS   = 'T')) AND
                       (E.CDCLIENTE    = C.CDCLIENTE)                                                          AND
                       (E.CDCONSUMIDOR = C.CDCONSUMIDOR)                                                       AND
                       ((INSTR(:CDFAMILISALD, '_'||E.CDFAMILISALD||'_') <> 0)   OR (:TCDFAMILISALD = 'T')) AND
                       (TO_DATE(E.DTMOVEXTCONS) < :DTEXTRATOINI)
                    GROUP BY
                       E.CDCLIENTE,
                       E.CDCONSUMIDOR,
                       E.CDFAMILISALD
                   ) D
                WHERE
                   (E.CDCLIENTE    = D.CDCLIENTE)    AND
                   (E.CDCONSUMIDOR = D.CDCONSUMIDOR) AND
                   (E.CDFAMILISALD = D.CDFAMILISALD) AND
                   (E.DTMOVEXTCONS = D.DTMOVEXTCONS)
                GROUP BY
                   E.CDCLIENTE,
                   E.CDCONSUMIDOR,
                   E.CDFAMILISALD,
                   E.DTMOVEXTCONS
               ) N
            WHERE
               (E.CDCLIENTE    = N.CDCLIENTE)    AND
               (E.CDCONSUMIDOR = N.CDCONSUMIDOR) AND
               (E.CDFAMILISALD = N.CDFAMILISALD) AND
               (E.DTMOVEXTCONS = N.DTMOVEXTCONS) AND
               (E.NRSEQMOVEXT  = N.NRSEQMOVEXT)
           ) SLDOA
        WHERE
           (C.CDCLIENTE    = :CDCLIENTE   ) AND
           ((INSTR(:CDCONSUMIDOR, '_'||C.CDCONSUMIDOR||'_' ) <> 0)  OR (:TCDCONSUMIDOR = 'T')) AND
           ((INSTR(:CDTIPOCONS  , '_'||C.CDTIPOCONS||'_'   ) <> 0)  OR (:TCDTIPOCONS   = 'T')) AND
           (TO_DATE(EX.DTMOVEXTCONS) BETWEEN :DTEXTRATOINI AND :DTEXTRATOFIN) AND
           (EX.CDCLIENTE     = C.CDCLIENTE  ) AND
           (EX.CDCLIENTE     = CLI.CDCLIENTE  ) AND
           (EX.CDCONSUMIDOR  = C.CDCONSUMIDOR ) AND
           ((INSTR(:CDFAMILISALD, '_'||EX.CDFAMILISALD||'_') <> 0)  OR (:TCDFAMILISALD = 'T')) AND
           (EX.CDFAMILISALD  = FS.CDFAMILISALD) AND
           (C.CDTIPOCONS     = T.CDTIPOCONS   ) AND
           (EX.CDCONSUMIDOR  = SLDOA.CDCONSUMIDOR (+)) AND
           (EX.CDFAMILISALD  = SLDOA.CDFAMILISALD (+)) AND
           (C.CDCLIENTE      = TC.CDCLIENTE(+))   AND
           (C.CDTIPOCONS     = TC.CDTIPOCONS(+))  AND
           (VE.CDSERVFILI   = SE.CDSERVFILI  (+)) AND
           (VE.CDFILIAL     = SE.CDFILIAL    (+)) AND
           (EX.CDCLIENTE    = VE.CDCLIENTE   (+)) AND
           (EX.CDCONSUMIDOR = VE.CDCONSUMIDOR(+)) AND
           (EX.CDFILIAL     = VE.CDFILIAL    (+)) AND
           (EX.CDCAIXA      = VE.CDCAIXA     (+)) AND
           (EX.NRSEQVENDA   = VE.NRSEQVENDA  (+)) AND
           ((EX.IDTPMOVEXT = 'D') and
            (UPPER(DSOPEEXTCONS) like 'ESTORNO%'))
        ORDER BY
           T.CDTIPOCONS, C.CDIDCONSUMID, FS.CDFAMILISALD , EX.DTMOVEXTCONS, EX.NRSEQMOVEXT
        ";
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        
        return $std->fetchAll();
    }
}