<?php

namespace Teknisa\MAN\Services;

use Doctrine\DBAL\Connection;
use Teknisa\MAN\Utilities;
use Doctrine\ORM\EntityManager;
use \Exception;

class ParaTerminalConsultaService {

    protected $entityManager;

    public function __construct(EntityManager $entityManager) {
        $this->entityManager = $entityManager;
    }

    public function findCaixas($row){
        $data = [];
        if (isset($row['CDFILIAL'])) {
            $params = array(
                ':CDFILIAL'     => $row['CDFILIAL']
            );
            
            $data = $this->entityManager->getConnection()->fetchAll(self::GET_CAIXAS, $params);
        }
        
        
        return $data;
    }
    
    public function findProdFili ($row) {
        $data = [];
        if (isset($row['CDFILIAL'])) {
            $params = array(
                ':CDFILIAL'     => $row['CDFILIAL']
            );
            
            $data = $this->entityManager->getConnection()->fetchAll(self::GET_PROD_FILI, $params);
        }
        
        return $data;
    }
    
    public function saveAll ($row) {
         $requiredFields = array(
            'CDFILIAL', 'CDCAIXA', 'DSMSGADITOT', 'DSMSGCONTOT',
            'IDTIPEXTTOT', 'IDEXIMSGSAL', 'IDSELLOJTOT', 'NRDIAINITOT',
            'NRDIAFINTOT', 'NRSEGFECTOT', 'IDPERCONSEXT', 'IDPERIMPREXT',
            'IDEXIEXTDIAS', 'IDIMPEXTDIAS', 'DSMSGAUTOPAG', 'DSMSGLIBPAG',
            'DSMSGPENPAG'
        );
    
        foreach ($requiredFields as $field) {
            if (!isset($row[$field])) {
                throw new \Exception("Required field $field is missing.");
            }
        }
        
        $params = array(
            ':CDFILIAL'     => $row['CDFILIAL'],
            ':CDCAIXA'      => $row['CDCAIXA'],
            ':DSMSGADITOT'  => $row['DSMSGADITOT'],
            ':DSMSGCONTOT'  => $row['DSMSGCONTOT'],
            ':IDTIPEXTTOT'  => $row['IDTIPEXTTOT'],
            ':IDEXIMSGSAL'  => $row['IDEXIMSGSAL'],
            ':IDSELLOJTOT'  => $row['IDSELLOJTOT'],
            ':NRDIAINITOT'  => $row['NRDIAINITOT'],
            ':NRDIAFINTOT'  => $row['NRDIAFINTOT'],
            ':NRSEGFECTOT'  => $row['NRSEGFECTOT'],
            ':IDPERCONSEXT' => $row['IDPERCONSEXT'],
            ':IDPERIMPREXT' => $row['IDPERIMPREXT'],
            ':IDEXIEXTDIAS' => $row['IDEXIEXTDIAS'],
            ':IDIMPEXTDIAS' => $row['IDIMPEXTDIAS'],
            ':DSMSGAUTOPAG' => $row['DSMSGAUTOPAG'],
            ':DSMSGLIBPAG'  => $row['DSMSGLIBPAG'],
            ':DSMSGPENPAG'  => $row['DSMSGPENPAG']
        );
    
        $this->entityManager->getConnection()->executeQuery(self::UPDATE_CAIXA, $params);
    }
    
    public function saveProdFili ($row) {
        $requiredFields = array(
            'CDFILIAL', 'CDPRODUTO', 'IDEXIBEPRODTCON'
        );
        
        foreach ($requiredFields as $field) {
            if (!isset($row[$field])) {
                throw new \Exception("Required field $field is missing.");
            }
        }

        $params = array (
            ':CDFILIAL'     => $row['CDFILIAL'],
            ':CDPRODUTO'     => $row['CDPRODUTO'],
            ':IDEXIBEPRODTCON'     => $row['IDEXIBEPRODTCON']
        );
        
        $this->entityManager->getConnection()->executeQuery(self::SAVE_PROD_FILI, $params);
    }
    
    public function editProdFili ($row) {
        $requiredFields = array(
            'CDFILIAL', 'CDPRODUTO', 'IDEXIBEPRODTCON'
        );
        
        foreach ($requiredFields as $field) {
            if (!isset($row[$field])) {
                throw new \Exception("Required field $field is missing.");
            }
        }
        
        $params = array (
            ':CDFILIAL'     => $row['CDFILIAL'],
            ':CDPRODUTO'     => $row['CDPRODUTO'],
            ':IDEXIBEPRODTCON'     => $row['IDEXIBEPRODTCON']
        );
        
        $this->entityManager->getConnection()->executeQuery(self::EDIT_PROD_FILI, $params);
    }
    
     public function deleteProdFili ($row) {
        $requiredFields = array(
            'CDFILIAL', 'CDPRODUTO'
        );
        
        foreach ($requiredFields as $field) {
            if (!isset($row[$field])) {
                throw new \Exception("Required field $field is missing.");
            }
        }
        
        $params = array (
            ':CDFILIAL'     => $row['CDFILIAL'],
            ':CDPRODUTO'     => $row['CDPRODUTO']
        );
        
        $this->entityManager->getConnection()->executeQuery(self::DEL_PROD_FILI, $params);
    }
    
    const GET_CAIXAS = "
        SELECT
           CDFILIAL    , CDCAIXA     , DSMSGADITOT , DSMSGCONTOT ,
           IDTIPEXTTOT , IDEXIMSGSAL , IDSELLOJTOT , NRDIAINITOT ,
           NRDIAFINTOT , NRSEGFECTOT , IDPERCONSEXT, IDPERIMPREXT,
           IDEXIEXTDIAS, IDIMPEXTDIAS, NMCAIXA     , DSMSGAUTOPAG,
           DSMSGLIBPAG , DSMSGPENPAG
        FROM
           CAIXA
        WHERE
           CDFILIAL = :CDFILIAL
        ORDER BY
           CDFILIAL, CDCAIXA
    ";
    
    const UPDATE_CAIXA = "
        UPDATE
           CAIXA
        SET
           DSMSGADITOT  = :DSMSGADITOT,
           DSMSGCONTOT  = :DSMSGCONTOT,
           IDTIPEXTTOT  = :IDTIPEXTTOT,
           IDEXIMSGSAL  = :IDEXIMSGSAL,
           IDSELLOJTOT  = :IDSELLOJTOT,
           NRDIAINITOT  = :NRDIAINITOT,
           NRDIAFINTOT  = :NRDIAFINTOT,
           NRSEGFECTOT  = :NRSEGFECTOT,
           IDPERCONSEXT = :IDPERCONSEXT,
           IDPERIMPREXT = :IDPERIMPREXT,
           IDEXIEXTDIAS = :IDEXIEXTDIAS,
           IDIMPEXTDIAS = :IDIMPEXTDIAS,
           DSMSGAUTOPAG = :DSMSGAUTOPAG,
           DSMSGLIBPAG  = :DSMSGLIBPAG ,
           DSMSGPENPAG  = :DSMSGPENPAG 
        WHERE
           (CDFILIAL = :CDFILIAL) AND
           (CDCAIXA  = :CDCAIXA )
   ";
   
   const GET_PROD_FILI = "
        SELECT P.CDFILIAL, P.CDPRODUTO, R.NMPRODUTO,
               R.CDARVPROD, P.IDEXIBEPRODTCON
          FROM PARAMPRODFILI P, PRODUTO R
        WHERE P.CDFILIAL = :CDFILIAL
           AND P.CDPRODUTO = R.CDPRODUTO
        ORDER BY P.CDFILIAL, R.CDARVPROD
   ";
   
   const SAVE_PROD_FILI = "
        INSERT INTO PARAMPRODFILI
            (CDFILIAL, CDPRODUTO, IDEXIBEPRODTCON)
        VALUES
            (:CDFILIAL, :CDPRODUTO, :IDEXIBEPRODTCON)
   ";
   
   const EDIT_PROD_FILI = "
        UPDATE PARAMPRODFILI
        SET
           IDEXIBEPRODTCON  = :IDEXIBEPRODTCON
        WHERE
           CDFILIAL = :CDFILIAL
           AND CDPRODUTO = :CDPRODUTO
   ";
   
   const DEL_PROD_FILI = "
        DELETE FROM PARAMPRODFILI
        WHERE CDFILIAL = :CDFILIAL 
            AND CDPRODUTO = :CDPRODUTO
   ";
}